@@ -1,13 +1,13 @@
 ;;; gnosis.el --- Spaced Repetition System  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2023-2024  Thanos Apollo
+;; Copyright (C) 2023-2026  Thanos Apollo
 
 ;; Author: Thanos Apollo <public@thanosapollo.org>
 ;; Keywords: extensions
 ;; URL: https://thanosapollo.org/projects/gnosis
 
-;; Package-Version: 20260114.454
-;; Package-Revision: 6bd8f4e5a843
+;; Package-Version: 20260202.352
+;; Package-Revision: 729d1ebf8deb
 
 ;; Package-Requires: ((emacs "27.2") (emacsql "4.1.0") (compat "29.1.4.2") (transient "0.7.2") (org-gnosis "0.0.9"))
 
@@ -67,10 +67,6 @@
 (unless (file-directory-p gnosis-dir)
   (make-directory gnosis-dir))
 
-(defcustom gnosis-cloze-string "[...]"
-  "Gnosis string to represent a cloze."
-  :type 'string)
-
 (defcustom gnosis-string-difference 1
   "Threshold value for string comparison in Gnosis.
 
@@ -153,6 +149,8 @@ When nil, review new themata last."
 
 (autoload 'gnosis-dashboard "gnosis-dashboard" nil t)
 
+(defvar gnosis-cloze-string "(...)")
+
 (defvar gnosis-testing nil
   "Change this to non-nil when running manual tests.")
 
@@ -299,14 +297,14 @@ When VERIFICATION is non-nil, skip `y-or-n-p' prompt."
                 (cl-rotatef (nth (1- i) seq) (nth j seq)))  ; Swap elements.
            finally return seq))
 
-(defun gnosis-completing-read (prompt seq)
+(defun gnosis-completing-read (prompt seq &optional require-match)
   "Call `gnosis-completing-read-function' with shuffled SEQ.
 
 PROMPT: Prompt for `gnosis-completing-read-function'
 History is disabled."
   (let ((history-add-new-input nil))
     (funcall gnosis-completing-read-function prompt
-	     (gnosis-shuffle (copy-sequence seq)))))
+	     (gnosis-shuffle (copy-sequence seq)) nil require-match)))
 
 (defun gnosis-insert-separator ()
   "Insert a dashed line spanning the entire width of the buffer."
@@ -430,7 +428,7 @@ This will not be applied to sentences that start with double space."
 	       (when (and hint (not (string-empty-p hint)) (not (string= hint "nil"))
 			  (not (string= "\"\"" hint))
 			  (search-backward cloze-string nil t))
-                 (replace-match (propertize (format "[%s]" hint)
+                 (replace-match (propertize (format "(%s)" hint)
 					    'face 'gnosis-face-cloze))
                  (goto-char (match-end 0)))) ; Move point to end of match
       (buffer-string))))
@@ -884,6 +882,41 @@ START is the search starting position, used internally for recursion."
               (gnosis-extract-id-links input (match-end 0)))
       nil)))
 
+(defun gnosis-get-linked-nodes (id)
+  "Return the title of linked org-gnosis node(s) for thema ID."
+  (let* ((links (gnosis-select 'dest 'links `(= source ,id) t))
+	 (org-gnosis-nodes (cl-loop for node-id in links
+				    collect (org-gnosis-select 'title 'nodes `(= id ,node-id) t))))
+    (and links (apply #'append org-gnosis-nodes))))
+
+(defun gnosis-view-linked-node (id)
+  "Visit linked node(s) for thema ID."
+  (let* ((node (gnosis-completing-read "Select node: " (gnosis-get-linked-nodes id) t)))
+    (window-configuration-to-register :gnosis-link-view)
+    (org-gnosis-find node)
+    (gnosis-link-view-mode)))
+
+(defun gnosis-link-view--exit ()
+  "Exit link view mode."
+  (interactive nil gnosis-link-view-mode)
+  (gnosis-link-view-mode -1)
+  (jump-to-register :gnosis-link-view)
+  (exit-recursive-edit))
+
+(defvar-keymap gnosis-link-view-mode-map
+  :doc "Keymap for `gnosis-link-view-mode'."
+  "C-c C-c" #'gnosis-link-view--exit)
+
+(define-minor-mode gnosis-link-view-mode "Gnosis Link View."
+  :interactive nil
+  :lighter " Gnosis Link View"
+  :keymap gnosis-link-view-mode-map
+  (if gnosis-link-view-mode
+      (setq-local header-line-format
+		  (substitute-command-keys
+		   " Return to review with: \\[gnosis-link-view--exit]"))
+    (setq-local header-line-format nil)))
+
 ;; TODO: Rewrite this! Tags should be an input of strings,
 ;; interactive handling should be done by "helper" funcs
 (cl-defun gnosis-collect-thema-ids (&key (tags nil) (due nil) (deck nil) (query nil))
@@ -969,13 +1002,6 @@ Optionally, provide THEMA-IDS of which the overdue ones will be returned."
 	   when (not (equal (cadr thema) (gnosis-algorithm-date)))
 	   collect (car thema)))
 
-(defun gnosis-review-get-due-themata--no-overdue (&optional thema-ids)
-  "Return due themata, without overdue.
-
-Optionally, provide a list for due THEMA-IDS."
-  (let ((thema-ids (or thema-ids (length (gnosis-review-get-due-themata)))))
-    (cl-set-difference thema-ids (gnosis-review-get-overdue-themata thema-ids))))
-
 (defun gnosis-review-last-interval (id)
   "Return last review interval for thema ID."
   (let* ((last-rev (gnosis-get 'last-rev 'review-log `(= id ,id)))
@@ -1207,18 +1233,30 @@ If NEW? is non-nil, increment new themata log by 1."
   (gnosis-review #'gnosis-monkeytype-session))
 
 (defun gnosis-monkeytype-thema (thema)
-  "Process monkeytyping for THEMA id."
+  "Process monkeytyping for THEMA id.
+
+This is used to type the keimenon of thema, with the answers highlighted.
+To monkeytype only the wrong answers use `gnosis-monkeytype-answer'."
   (let* ((thema-context (gnosis-select '[keimenon type answer] 'themata `(= id ,thema) t))
 	 (keimenon (replace-regexp-in-string
 		    "\\[\\[\\([^]]+\\)\\]\\[\\([^]]+\\)\\]\\]" "\\2" ;; remove links
 		    (nth 0 thema-context)))
 	 (type (nth 1 thema-context))
-	 (answer (nth 2 thema-context)))
+	 (answer (cl-loop for answer in (nth 2 thema-context)
+			  collect (gnosis-utils-trim-quotes answer))))
     (cond ((string= type "basic")
 	   (gnosis-monkeytype (concat keimenon "\n" (car answer)) type
 			      answer))
 	  (t (gnosis-monkeytype keimenon type answer)))))
 
+(defun gnosis-monkeytype-answer (thema)
+  "Monkeytype answer for THEMA id."
+  (let* ((thema-context (gnosis-select '[type answer] 'themata `(= id ,thema) t))
+	 (type (nth 0 thema-context))
+	 (answer (cl-loop for answer in (nth 1 thema-context)
+			  collect (gnosis-utils-trim-quotes answer))))
+    (gnosis-monkeytype (mapconcat #'identity answer " ") type answer)))
+
 (defun gnosis-review-process-thema (thema &optional thema-count)
   "Process review for THEMA and update session statistics.
 
@@ -1229,10 +1267,12 @@ This is a helper function for `gnosis-review-session'."
   (let ((success (gnosis-review--display-thema thema))
 	(thema-count (or thema-count 0)))
     (cl-incf thema-count)
-    (unless success (gnosis-monkeytype-thema thema))
+    (unless success (gnosis-monkeytype-answer thema))
     (gnosis-review-actions success thema thema-count)
-    (jump-to-register :gnosis-pre-image)
-    (gnosis-review-update-header thema-count)
+    ;; Use jump-to-register after first review.
+    (and (not (null (get-register :gnosis-pre-image))) (jump-to-register :gnosis-pre-image))
+    (setq gnosis-review-themata (remove thema gnosis-review-themata))
+    (gnosis-review-update-header thema-count (length gnosis-review-themata))
     thema-count))
 
 (defun gnosis-review-update-header (reviewed-count &optional remaining-reviews)
@@ -1242,7 +1282,7 @@ REVIEWED-COUNT: Total number of items that have been reviewed in
 current session.
 REMAINING-REVIEWS: Total number of remaining items to be reviewed."
   (with-current-buffer (get-buffer-create gnosis-review-buffer-name)
-    (let ((remaining-reviews (or remaining-reviews gnosis-due-themata-total)))
+    (let ((remaining-reviews (or remaining-reviews (1+ (length gnosis-review-themata)))))
       (setq-local header-line-format
                   (gnosis-center-string
 		   (format "%s %s %s"
@@ -1333,6 +1373,15 @@ be called with new SUCCESS value plus THEMA & THEMA-COUNT."
   (gnosis-display-next-review thema success)
   (gnosis-review-actions success thema thema-count))
 
+(defun gnosis-review-action--view-link (success thema thema-count)
+  "View linked node(s) for THEMA."
+  (if (gnosis-get-linked-nodes thema)
+    (progn (gnosis-view-linked-node thema)
+	   (recursive-edit))
+    (message (format "No linked nodes for thema: %d" thema))
+    (sleep-for 0.5))
+  (gnosis-review-actions success thema thema-count))
+
 (defun gnosis-review-actions (success id thema-count)
   "Specify action during review of thema.
 
@@ -1342,17 +1391,20 @@ THEMA-COUNT: Total themata reviewed
 
 To customize the keybindings, adjust `gnosis-review-keybindings'."
   (let* ((prompt
-	  "Action: %sext gnosis, %sverride result, %suspend thema, %sdit thema, %suit: ")
+	  "Action: %sext, %sverride result, %suspend, %selete, %sdit thema, %siew link, %suit: ")
 	 (choice (read-char-choice
 		  (apply #'format prompt
 			 (mapcar
-			  (lambda (str) (propertize str 'face 'match)) '("n" "o" "s" "e" "q")))
-		  '(?n ?o ?s ?e ?q))))
+			  (lambda (str) (propertize str 'face 'match))
+			  '("n" "o" "s" "d" "e" "v" "q")))
+		  '(?n ?o ?s ?d ?e ?v ?q))))
     (pcase choice
       (?n (gnosis-review-result id success))
       (?o (gnosis-review-action--override success id thema-count))
       (?s (gnosis-review-action--suspend success id thema-count))
+      (?d (gnosis-delete-thema id))
       (?e (gnosis-review-action--edit success id thema-count))
+      (?v (gnosis-review-action--view-link success id thema-count))
       (?q (gnosis-review-action--quit success id)))))
 
 ;;;###autoload
@@ -1375,7 +1427,8 @@ FN: Review function, defaults to `gnosis-review-session'"
       ("Overdue themata"
        (funcall fn (gnosis-review-get-overdue-themata)))
       ("Due themata (Without Overdue)"
-       (funcall fn (gnosis-review-get-due-themata--no-overdue)))
+       (funcall fn (cl-set-difference (mapcar #'car (gnosis-review-get--due-themata))
+				      (gnosis-review-get-overdue-themata))))
       ("All themata of deck"
        (funcall fn (gnosis-collect-thema-ids :deck (gnosis--get-deck-id))))
       ("All themata of tag(s)"
@@ -1902,6 +1955,7 @@ generate new thema id."
   :doc "gnosis org mode map"
   "C-c C-c" #'gnosis-save
   "C-c C-q" #'gnosis-tags-prompt
+  "C-c C-o" #'org-gnosis-goto-id
   "C-c C-k" #'gnosis-edit-quit)
 
 (define-derived-mode gnosis-edit-mode org-mode "Gnosis Org"
